package org.springframework.boot.loader.jar;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;
import org.springframework.boot.loader.data.RandomAccessData;
import org.springframework.boot.loader.jar.AsciiBytes;
import org.springframework.boot.loader.jar.Bytes;
import org.springframework.boot.loader.jar.CentralDirectoryEndRecord;
import org.springframework.boot.loader.jar.CentralDirectoryFileHeader;
import org.springframework.boot.loader.jar.CentralDirectoryVisitor;
import org.springframework.boot.loader.jar.FileHeader;
import org.springframework.boot.loader.jar.JarEntry;
import org.springframework.boot.loader.jar.JarEntryCertification;
import org.springframework.boot.loader.jar.JarEntryFilter;
import org.springframework.boot.loader.jar.JarFile;
import org.springframework.boot.loader.jar.ZipInflaterInputStream;

class JarFileEntries implements CentralDirectoryVisitor, Iterable<JarEntry> {
  static {
    int version;
  }
  
  private static final Runnable NO_VALIDATION = () -> {
    
    };
  
  private static final String META_INF_PREFIX = "META-INF/";
  
  private static final Attributes.Name MULTI_RELEASE = new Attributes.Name("Multi-Release");
  
  private static final int BASE_VERSION = 8;
  
  private static final int RUNTIME_VERSION;
  
  private static final long LOCAL_FILE_HEADER_SIZE = 30L;
  
  private static final char SLASH = '/';
  
  private static final char NO_SUFFIX = '\000';
  
  protected static final int ENTRY_CACHE_SIZE = 25;
  
  private final JarFile jarFile;
  
  private final JarEntryFilter filter;
  
  private RandomAccessData centralDirectoryData;
  
  private int size;
  
  private int[] hashCodes;
  
  private Offsets centralDirectoryOffsets;
  
  private int[] positions;
  
  private Boolean multiReleaseJar;
  
  private JarEntryCertification[] certifications;
  
  static {
    try {
      Object runtimeVersion = Runtime.class.getMethod("version", new Class[0]).invoke(null, new Object[0]);
      version = ((Integer)runtimeVersion.getClass().getMethod("major", new Class[0]).invoke(runtimeVersion, new Object[0])).intValue();
    } catch (Throwable ex) {
      version = 8;
    } 
    RUNTIME_VERSION = version;
  }
  
  private final Map<Integer, FileHeader> entriesCache = Collections.synchronizedMap((Map<Integer, FileHeader>)new Object(this, 16, 0.75F, true));
  
  JarFileEntries(JarFile jarFile, JarEntryFilter filter) {
    this.jarFile = jarFile;
    this.filter = filter;
    if (RUNTIME_VERSION == 8)
      this.multiReleaseJar = Boolean.valueOf(false); 
  }
  
  public void visitStart(CentralDirectoryEndRecord endRecord, RandomAccessData centralDirectoryData) {
    int maxSize = endRecord.getNumberOfRecords();
    this.centralDirectoryData = centralDirectoryData;
    this.hashCodes = new int[maxSize];
    this.centralDirectoryOffsets = Offsets.from(endRecord);
    this.positions = new int[maxSize];
  }
  
  public void visitFileHeader(CentralDirectoryFileHeader fileHeader, long dataOffset) {
    AsciiBytes name = applyFilter(fileHeader.getName());
    if (name != null)
      add(name, dataOffset); 
  }
  
  private void add(AsciiBytes name, long dataOffset) {
    this.hashCodes[this.size] = name.hashCode();
    this.centralDirectoryOffsets.set(this.size, dataOffset);
    this.positions[this.size] = this.size;
    this.size++;
  }
  
  public void visitEnd() {
    sort(0, this.size - 1);
    int[] positions = this.positions;
    this.positions = new int[positions.length];
    for (int i = 0; i < this.size; i++)
      this.positions[positions[i]] = i; 
  }
  
  int getSize() {
    return this.size;
  }
  
  private void sort(int left, int right) {
    if (left < right) {
      int pivot = this.hashCodes[left + (right - left) / 2];
      int i = left;
      int j = right;
      while (i <= j) {
        while (this.hashCodes[i] < pivot)
          i++; 
        while (this.hashCodes[j] > pivot)
          j--; 
        if (i <= j) {
          swap(i, j);
          i++;
          j--;
        } 
      } 
      if (left < j)
        sort(left, j); 
      if (right > i)
        sort(i, right); 
    } 
  }
  
  private void swap(int i, int j) {
    swap(this.hashCodes, i, j);
    this.centralDirectoryOffsets.swap(i, j);
    swap(this.positions, i, j);
  }
  
  public Iterator<JarEntry> iterator() {
    return (Iterator<JarEntry>)new EntryIterator(this, NO_VALIDATION, null);
  }
  
  Iterator<JarEntry> iterator(Runnable validator) {
    return (Iterator<JarEntry>)new EntryIterator(this, validator, null);
  }
  
  boolean containsEntry(CharSequence name) {
    return (getEntry(name, FileHeader.class, true) != null);
  }
  
  JarEntry getEntry(CharSequence name) {
    return getEntry(name, JarEntry.class, true);
  }
  
  InputStream getInputStream(String name) throws IOException {
    FileHeader entry = getEntry(name, FileHeader.class, false);
    return getInputStream(entry);
  }
  
  InputStream getInputStream(FileHeader entry) throws IOException {
    ZipInflaterInputStream zipInflaterInputStream;
    if (entry == null)
      return null; 
    InputStream inputStream = getEntryData(entry).getInputStream();
    if (entry.getMethod() == 8)
      zipInflaterInputStream = new ZipInflaterInputStream(inputStream, (int)entry.getSize()); 
    return (InputStream)zipInflaterInputStream;
  }
  
  RandomAccessData getEntryData(String name) throws IOException {
    FileHeader entry = getEntry(name, FileHeader.class, false);
    if (entry == null)
      return null; 
    return getEntryData(entry);
  }
  
  private RandomAccessData getEntryData(FileHeader entry) throws IOException {
    RandomAccessData data = this.jarFile.getData();
    byte[] localHeader = data.read(entry.getLocalHeaderOffset(), 30L);
    long nameLength = Bytes.littleEndianValue(localHeader, 26, 2);
    long extraLength = Bytes.littleEndianValue(localHeader, 28, 2);
    return data.getSubsection(entry.getLocalHeaderOffset() + 30L + nameLength + extraLength, entry
        .getCompressedSize());
  }
  
  private <T extends FileHeader> T getEntry(CharSequence name, Class<T> type, boolean cacheEntry) {
    T entry = doGetEntry(name, type, cacheEntry, null);
    if (!isMetaInfEntry(name) && isMultiReleaseJar()) {
      int version = RUNTIME_VERSION;
      AsciiBytes nameAlias = (entry instanceof JarEntry) ? ((JarEntry)entry).getAsciiBytesName() : new AsciiBytes(name.toString());
      while (version > 8) {
        T versionedEntry = doGetEntry("META-INF/versions/" + version + "/" + name, type, cacheEntry, nameAlias);
        if (versionedEntry != null)
          return versionedEntry; 
        version--;
      } 
    } 
    return entry;
  }
  
  private boolean isMetaInfEntry(CharSequence name) {
    return name.toString().startsWith("META-INF/");
  }
  
  private boolean isMultiReleaseJar() {
    Boolean multiRelease = this.multiReleaseJar;
    if (multiRelease != null)
      return multiRelease.booleanValue(); 
    try {
      Manifest manifest = this.jarFile.getManifest();
      if (manifest == null) {
        multiRelease = Boolean.valueOf(false);
      } else {
        Attributes attributes = manifest.getMainAttributes();
        multiRelease = Boolean.valueOf(attributes.containsKey(MULTI_RELEASE));
      } 
    } catch (IOException ex) {
      multiRelease = Boolean.valueOf(false);
    } 
    this.multiReleaseJar = multiRelease;
    return multiRelease.booleanValue();
  }
  
  private <T extends FileHeader> T doGetEntry(CharSequence name, Class<T> type, boolean cacheEntry, AsciiBytes nameAlias) {
    int hashCode = AsciiBytes.hashCode(name);
    T entry = getEntry(hashCode, name, false, type, cacheEntry, nameAlias);
    if (entry == null) {
      hashCode = AsciiBytes.hashCode(hashCode, '/');
      entry = getEntry(hashCode, name, '/', type, cacheEntry, nameAlias);
    } 
    return entry;
  }
  
  private <T extends FileHeader> T getEntry(int hashCode, CharSequence name, char suffix, Class<T> type, boolean cacheEntry, AsciiBytes nameAlias) {
    int index = getFirstIndex(hashCode);
    while (index >= 0 && index < this.size && this.hashCodes[index] == hashCode) {
      T entry = getEntry(index, type, cacheEntry, nameAlias);
      if (entry.hasName(name, suffix))
        return entry; 
      index++;
    } 
    return null;
  }
  
  private <T extends FileHeader> T getEntry(int index, Class<T> type, boolean cacheEntry, AsciiBytes nameAlias) {
    try {
      JarEntry jarEntry;
      long offset = this.centralDirectoryOffsets.get(index);
      FileHeader cached = this.entriesCache.get(Integer.valueOf(index));
      FileHeader entry = (cached != null) ? cached : (FileHeader)CentralDirectoryFileHeader.fromRandomAccessData(this.centralDirectoryData, offset, this.filter);
      if (CentralDirectoryFileHeader.class.equals(entry.getClass()) && type.equals(JarEntry.class))
        jarEntry = new JarEntry(this.jarFile, index, (CentralDirectoryFileHeader)entry, nameAlias); 
      if (cacheEntry && cached != jarEntry)
        this.entriesCache.put(Integer.valueOf(index), jarEntry); 
      return (T)jarEntry;
    } catch (IOException ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  private int getFirstIndex(int hashCode) {
    int index = Arrays.binarySearch(this.hashCodes, 0, this.size, hashCode);
    if (index < 0)
      return -1; 
    while (index > 0 && this.hashCodes[index - 1] == hashCode)
      index--; 
    return index;
  }
  
  void clearCache() {
    this.entriesCache.clear();
  }
  
  private AsciiBytes applyFilter(AsciiBytes name) {
    return (this.filter != null) ? this.filter.apply(name) : name;
  }
  
  JarEntryCertification getCertification(JarEntry entry) throws IOException {
    JarEntryCertification[] certifications = this.certifications;
    if (certifications == null) {
      certifications = new JarEntryCertification[this.size];
      try (JarInputStream certifiedJarStream = new JarInputStream(this.jarFile.getData().getInputStream())) {
        JarEntry certifiedEntry = null;
        while ((certifiedEntry = certifiedJarStream.getNextJarEntry()) != null) {
          certifiedJarStream.closeEntry();
          int index = getEntryIndex(certifiedEntry.getName());
          if (index != -1)
            certifications[index] = JarEntryCertification.from(certifiedEntry); 
        } 
      } 
      this.certifications = certifications;
    } 
    JarEntryCertification certification = certifications[entry.getIndex()];
    return (certification != null) ? certification : JarEntryCertification.NONE;
  }
  
  private int getEntryIndex(CharSequence name) {
    int hashCode = AsciiBytes.hashCode(name);
    int index = getFirstIndex(hashCode);
    while (index >= 0 && index < this.size && this.hashCodes[index] == hashCode) {
      FileHeader candidate = getEntry(index, FileHeader.class, false, null);
      if (candidate.hasName(name, false))
        return index; 
      index++;
    } 
    return -1;
  }
  
  private static void swap(int[] array, int i, int j) {
    int temp = array[i];
    array[i] = array[j];
    array[j] = temp;
  }
  
  private static void swap(long[] array, int i, int j) {
    long temp = array[i];
    array[i] = array[j];
    array[j] = temp;
  }
}
